#include <zephyr/drivers/adc.h>
#include "cm_read_analog.h"

const struct adc_channel_cfg adc_cfg;



#if !DT_NODE_EXISTS(DT_PATH(zephyr_user)) || \
	!DT_NODE_HAS_PROP(DT_PATH(zephyr_user), io_channels)
#error "No suitable devicetree overlay specified"
#endif

#define DT_SPEC_AND_COMMA(node_id, prop, idx) \
	ADC_DT_SPEC_GET_BY_IDX(node_id, idx),

/* Data of ADC io-channels specified in devicetree. */
static const struct adc_dt_spec adc_channels[] = {
	DT_FOREACH_PROP_ELEM(DT_PATH(zephyr_user), io_channels,
			     DT_SPEC_AND_COMMA)
};



// Get a single analog value from a source
message_error_t cm_read_analog_mv( analog_source_t src , int32_t *result )
{
	int err;
	int16_t buf;
    int32_t val_mv;

	struct adc_sequence sequence = {
		.buffer = &buf,
		/* buffer size in bytes, not number of samples */
		.buffer_size = sizeof(buf),
	};

	/* Configure channel prior to sampling. */
    if (!device_is_ready(adc_channels[src].dev)) {
        printk("ADC controller device not ready\n");
        return ADC_NOT_READY;
    }
    err = adc_channel_setup_dt(&adc_channels[src]);
    if (err < 0) {
        printk("Could not setup channel #%d (%d)\n", src, err);
        return ADC_NOT_READY;
    }
    (void)adc_sequence_init_dt(&adc_channels[src], &sequence);
    err = adc_read(adc_channels[src].dev, &sequence);
    if( err < 0 ) {
        printk("Cound not read channel %d\n",src);
        return ADC_READ_ERROR;
    } else {
        val_mv = buf;
        err = adc_raw_to_millivolts_dt(&adc_channels[src],&val_mv);
        if( err < 0 ) {
            printk("Channel %d reads code of 0x%x\n",src,val_mv);
        }
        // printk("Channel %d reads %dmV\n",src,val_mv);
    }
    *result = val_mv;
    return MESSAGE_OK;
}
