#include "cm_write_vout.h"
#include <zephyr/drivers/spi.h>
#include "cm_utils.h"

#if DT_NODE_HAS_STATUS(DT_NODELABEL(my_spi_master), okay )
#else
#warning "SPI is disabled"
#endif

#define MY_SPI_MASTER DT_NODELABEL(my_spi_master)
const struct device * spi_dev = DEVICE_DT_GET(MY_SPI_MASTER);

struct spi_cs_control spim_cs = {
	.gpio = SPI_CS_GPIOS_DT_SPEC_GET(DT_NODELABEL(reg_my_spi_master)),
	.delay = 0,
};
static const struct spi_config spi_cfg = {
	.operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB |
				 SPI_MODE_CPOL | SPI_MODE_CPHA,
	.frequency = 1000000,
	.slave = 0,
	.cs = &spim_cs,
};

static void swap_bytes( uint8_t * bytes , uint16_t len ) {
    uint8_t temp;
    if( len == 2 ) {
        temp = bytes[0];
        bytes[0] = bytes[1];
        bytes[1] = temp;
    }
}

static message_error_t write_mcp4911_spi( uint16_t dac_code )
{
    if( dac_code > 0X3FF ) {
        printk("Value too big for vout DAC: %d\n",dac_code);
        return DATA_OUT_OF_BOUNDS;
    }
    union dac_combo {
        struct {
            // LSB first in this struct
            uint16_t dont_care: 2;
            uint16_t code: 10;
            uint16_t shdn: 1;
            uint16_t ga: 1;
            uint16_t buf: 1;
            uint16_t leading_zero: 1;
        } fields;
        uint8_t bytes[2];
    };

    union dac_combo spi_data = {
        .fields.leading_zero = 0,
        .fields.buf = 0, // unbuffered
        .fields.ga = 1, // gain = 1
        .fields.shdn = 1, // active mode
        .fields.code = dac_code
    };
    // printk("SPI (MSB first): 0x%x%x\n", spi_data.bytes[1] , spi_data.bytes[0] );
    // printk("SPI data (MSB first): " PRINTF_BINARY_PATTERN_INT8 PRINTF_BINARY_PATTERN_INT8 "\n",
    //  PRINTF_BYTE_TO_BINARY_INT8((spi_data.bytes[1])), PRINTF_BYTE_TO_BINARY_INT8((spi_data.bytes[0])) );
    swap_bytes( spi_data.bytes , 2 );
    const struct spi_buf my_spi_buf[1] = {// Buffer to hold one set of SPI data
        [0].buf = spi_data.bytes,// The byte array to send
        [0].len = sizeof(spi_data.bytes) // Number of bytes in buffer
    };
    const struct spi_buf_set tx_buff = {
        .buffers = my_spi_buf,
        .count = 1
    };
    if( spi_dev == NULL ) {
        printk("NULL SPI device\n");
        return SPI_NOT_INITIALIZED;
    }
	if(!device_is_ready(spi_dev)) {
		printk("SPI master device not ready!\n");
        return SPI_NOT_READY;
	}
	if(!device_is_ready(spim_cs.gpio.port)){
		printk("SPI master chip select device not ready!\n");
        return SPI_NOT_READY;
	}

    int spi_return;
    spi_return = spi_transceive(spi_dev,&spi_cfg,&tx_buff,NULL);
    if( spi_return == 0 ) {
        return MESSAGE_OK;
    } else {
        printk("Error number %d in SPI write.\n" , spi_return );
        return SPI_WRITE_ERROR;
    }
}

message_error_t cm_write_vout_mv( uint32_t vout_mv )
{
    // Do scaling to get DAC code
    float vout = vout_mv / 1000.0;
    uint16_t dac_code = (uint16_t) ((15.0 - vout)/0.014);
    printk("Writing DAC code 0x%x\n",dac_code);
    return write_mcp4911_spi(dac_code);
}
